# -*- coding=utf-8 -*-
from pytest_isort import FileIgnorer, IsortError


pytest_plugins = 'pytester',


def test_version():
    import pytest_isort
    assert pytest_isort.__version__


def test_file_ignorer(tmpdir):
    ignorer = FileIgnorer([
        'somefile.py',
        '# Commented line',
        'folder1/file1.py',
        'folder2/*  # With comment',
    ])

    assert ignorer.ignores == [
        'somefile.py',
        'folder1/file1.py',
        'folder2/*'
    ]

    assert ignorer(tmpdir.join('otherfile.py')) is False
    assert ignorer(tmpdir.join('folder1/file1.py')) == 'folder1/file1.py'
    assert ignorer(tmpdir.join('folder1/file2.py')) is False
    assert ignorer(tmpdir.join('folder2/file1.py')) == 'folder2/*'


class TestIsortError:
    def test_no_output(self):
        assert IsortError('').simplified_error() == ''

    def test_no_diff(self):
        expected = 'FOOBAR: Lorem ipsum 123'
        assert IsortError('FOOBAR: Lorem ipsum 123').simplified_error() == expected

    def test_with_diff(self):
        isort_output = """ERROR: /path/to/foobar.py Imports are incorrectly sorted.
            --- /path/to/foobar.py:before
            +++ /path/to/foobar.py:after
            @@ 123,123
            +import logging
             import os
             import tempfile
            -import logging"""

        expected_output = """ERROR: /path/to/foobar.py Imports are incorrectly sorted.

            +import logging
             import os
             import tempfile
            -import logging"""

        assert IsortError(isort_output).simplified_error() == expected_output


def test_file_no_ignored(testdir):
    testdir.tmpdir.ensure('file1.py')
    testdir.tmpdir.ensure('file2.py')

    result = testdir.runpytest('--isort')
    result.stdout.fnmatch_lines([
        'file1.py*',
        'file2.py*',
        '*2 passed*',
    ])
    assert result.ret == 0


def test_file_ignored(testdir):
    testdir.tmpdir.ensure('file1.py')
    testdir.tmpdir.ensure('file2.py')

    testdir.makeini("""
        [pytest]
        isort_ignore =
            file2.py
            file3.py
    """)

    result = testdir.runpytest('--isort')
    result.stdout.fnmatch_lines([
        'file1.py*',
        '*1 passed*',
    ])
    assert result.ret == 0

def test_correctly_sorted(testdir):
    test_file = testdir.makepyfile("""
        import os
        import sys
    """)

    # Ugly hack to append the missing newline.
    test_file = test_file.write(test_file.read() + '\n')

    result = testdir.runpytest('--isort', '--verbose')
    result.stdout.fnmatch_lines([
        '*test_correctly_sorted*PASSED*',
        '*1 passed*',
    ])
    assert result.ret == 0


def test_incorrectly_sorted(testdir):
    test_file = testdir.makepyfile("""
        import sys
        import os
    """)

    # Ugly hack to append the missing newline.
    test_file = test_file.write(test_file.read() + '\n')

    result = testdir.runpytest('--isort', '-vv')
    result.stdout.fnmatch_lines([
        '*test_incorrectly_sorted*FAILED*',
        '*FAILURES*',
        '*isort-check*',
        '*1 failed*',
    ])
    assert result.ret == 1



# -*- coding: utf-8 -*-
import os

import isort
import py
import pytest


__version__ = '0.3.1'


MTIMES_HISTKEY = 'isort/mtimes'


def pytest_configure(config):
    config.addinivalue_line(
        'markers',
        'isort: Test to check import ordering')


def pytest_addoption(parser):
    group = parser.getgroup('general')
    group.addoption('--isort', action='store_true', help=(
        'perform import ordering checks on .py files'))

    parser.addini('isort_ignore', type='linelist', help=(
        'each line specifies a glob filename pattern which will be ignored. '
        'Example: */__init__.py'
    ))


def pytest_sessionstart(session):
    config = session.config
    if config.option.isort:
        config._isort_mtimes = config.cache.get(MTIMES_HISTKEY, {})
        config._isort_ignore = FileIgnorer(config.getini('isort_ignore'))


def pytest_collect_file(path, parent):
    config = parent.config
    if config.option.isort and path.ext == '.py':
        if not config._isort_ignore(path):
            return IsortItem(path, parent)


def pytest_sessionfinish(session):
    config = session.config

    # isort might not be enabled, lets check if we have a mtimes dict.
    if hasattr(config, '_isort_mtimes'):
        config.cache.set(MTIMES_HISTKEY, config._isort_mtimes)


def isort_check_file(path):
    """
    Given a file path, this function executes the actual isort check.
    """

    sorter = isort.SortImports(str(path), check=True, show_diff=True)
    return sorter.incorrectly_sorted


class FileIgnorer:
    """
    This class helps to maintain a list of ignored filepaths.
    FileIgnorer parses the "isort_ignore" list from pytest.ini and provides an
    interface to check if a certain filepath should be ignored.

    Based on the Ignorer class from pytest-pep8.
    """

    def __init__(self, ignorelines):
        self.ignores = ignores = []

        for line in ignorelines:
            comment_position = line.find("#")
            # Strip comments.
            if comment_position != -1:
                line = line[:comment_position]

            glob = line.strip()

            # Skip blank lines.
            if not glob:
                continue

            # Normalize path if needed.
            if glob and os.sep != '/' and '/' in glob:
                glob = glob.replace('/', os.sep)

            ignores.append(glob)

    def __call__(self, path):
        """
        Given a filepath, returns wether the path is ignored or not.
        """

        for glob in self.ignores:
            if path.fnmatch(glob):
                return glob

        return False


class IsortError(Exception):
    """
    Indicates an error during isort checks.
    """

    def __init__(self, output=''):
        self.output = output

    def simplified_error(self):
        """
        This helper strips out unneeded diff "header" lines (+++, ---, @@).

        These lines are not needed in this case. In addition, this helper inserts
        a blank line between the error message and the diff output.
        """
        if not self.output:
            return ''

        valid_lines = [
            line
            for line in self.output.splitlines()
            if line.strip().split(' ', 1)[0] not in ('+++', '---', '@@')
        ]

        if len(valid_lines) > 1:
            valid_lines.insert(1, '')

        return '\n'.join(valid_lines)


class IsortItem(pytest.Item, pytest.File):
    """
    py.test Item to run the isort check.
    """

    def __init__(self, path, parent):
        super(IsortItem, self).__init__(path, parent)
        self._nodeid += "::ISORT"
        self.add_marker('isort')

    def setup(self):
        # Fetch mtime of file to compare with cache and for writing to cache
        # later on.
        self._mtime = self.fspath.mtime()

        old = self.config._isort_mtimes.get(str(self.fspath), 0)
        if old == self._mtime:
            pytest.skip('file(s) previously passed isort checks')

    def runtest(self):
        # Execute actual isort check.
        found_errors, stdout, stderr = py.io.StdCaptureFD.call(
            isort_check_file, self.fspath)

        if found_errors:
            # Strip diff header, this is not needed when displaying errors.
            raise IsortError(stdout)

        # Update mtime only if test passed otherwise failures
        # would not be re-run next time.
        self.config._isort_mtimes[str(self.fspath)] = self._mtime

    def repr_failure(self, excinfo):
        if excinfo.errisinstance(IsortError):
            # Return the simplified/filtered error output of isort.
            return excinfo.value.simplified_error()

        return super(IsortItem, self).repr_failure(excinfo)

    def reportinfo(self):
        return (self.fspath, -1, 'isort-check')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import codecs
from setuptools import setup


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


setup(
    name='pytest-isort',
    description='py.test plugin to check import ordering using isort',
    long_description=read('README.rst'),
    version='0.3.1',
    license='BSD',
    author='Moccu GmbH & Co. KG',
    author_email='info@moccu.com',
    url='http://github.com/moccu/pytest-isort/',
    py_modules=['pytest_isort'],
    entry_points={'pytest11': ['isort = pytest_isort']},
    test_suite='.',
    install_requires=[
        'pytest>=3.5',
        'isort>=4.0'
    ],
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
